@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2007  Robey Pointer <robeypointer@gmail.com>
 #
 # This file is part of paramiko.
 #
@@ -89,10 +89,11 @@ class SFTPFile (BufferedFile):
         k = [i for i in self._prefetch_reads if i[0] <= offset]
         if len(k) == 0:
             return False
-        k.sort(cmp=lambda x, y: cmp(x[0], y[0]))
+        k.sort(lambda x, y: cmp(x[0], y[0]))
         buf_offset, buf_size = k[-1]
         if buf_offset + buf_size <= offset:
-            return True
+            # prefetch request ends before this one begins
+            return False
         if buf_offset + buf_size >= offset + size:
             # inclusive
             return True
@@ -463,7 +464,6 @@ class SFTPFile (BufferedFile):
             raise SFTPError('Expected data')
         data = msg.get_string()
         offset, length = self._prefetch_reads.pop(0)
-        assert length == len(data)
         self._prefetch_data[offset] = data
         if len(self._prefetch_reads) == 0:
             self._prefetch_done = True
